/*
 * Tencent is pleased to support the open source community by making  XiaoweiSDK Demo Codes available.
 *
 * Copyright (C) 2017 THL A29 Limited, a Tencent company. All rights reserved.
 *
 * Licensed under the MIT License (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://opensource.org/licenses/MIT
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 *
 */
#ifndef AutoBuffer_hpp
#define AutoBuffer_hpp

#include <string>
#include <vector>

template <typename T>
class CAutoBuffer
{
  public:
    CAutoBuffer(size_t count)
    {
        buffer_ = new T[count];
    }
    ~CAutoBuffer()
    {
        if (buffer_)
            delete[] buffer_;
    }

    T *Get()
    {
        return buffer_;
    }

    T &operator[](size_t index)
    {
        return *(buffer_ + index);
    }

  private:
    CAutoBuffer();

    T *buffer_;
};

class CStringHolder
{
  public:
    ~CStringHolder();
    const char *Hold(const char *content);
    const char *Hold(const char *content, size_t length);

    void Drop(const char *holder);

  private:
    std::vector<std::string *> storage_;
};

#endif /* AutoBuffer_hpp */
